// GS2x shader - by guest(r)
// guest.r@gmail.com
//
// license: GNU-GPL
//
//
// ***DosBox***
//
// Best use it in the fullscreen mode (works with fullfixed=true, scaler=normal2x)
// the output should be set to direct3d.
//
// Best fullscreen modes are integer multiplies of the native "game" mode (like 2x(320,200), 4x(320,200)).
// The output image might also look nice with anything near (like 640x480, 1280x960...), looks nicer
// when aspect is set to false.


// matrix definitions and sampler init 

half4x4 World			: WORLD;
half4x4 View			: VIEW;
half4x4 Projection		: PROJECTION;
half4x4 WorldView		: WORLDVIEW;			
half4x4 ViewProjection		: VIEWPROJECTION;		
half4x4 WorldViewProjection	: WORLDVIEWPROJECTION;		

float2 ps  : TEXELSIZE = 1.0F/256.0F;

texture s_tex	: SOURCETEXTURE;

sampler	s_sam = sampler_state
{
 Texture   = (s_tex);
 MinFilter = POINT;
 MagFilter = POINT;
};



string name : NAME = "GS2x";

string combineTechique : COMBINETECHNIQUE =  "GS2x_SHADER";

// vertex shader linkin' structure

struct VERTEX_STUFF
{
  float4  coord : POSITION;
  float2  CT    : TEXCOORD0;
  float2  UL    : TEXCOORD1;
  float2  UR    : TEXCOORD2;
  float2  DL    : TEXCOORD3;
  float2  DR    : TEXCOORD4;
};


// **VS**

VERTEX_STUFF GS_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF OUT = (VERTEX_STUFF)0;
  
  float dx = ps.x*0.5;
  float dy = ps.y*0.5;

  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  OUT.UL = tc + float2(-dx,-dy);
  OUT.UR = tc + float2( dx,-dy);
  OUT.DL = tc + float2(-dx, dy);
  OUT.DR = tc + float2( dx, dy);
  return OUT;
}


// **PS**

float4 GS_FRAGMENT ( in VERTEX_STUFF VAR ) : COLOR

{
  half3 c11 = tex2D(s_sam, VAR.CT).xyz;
  half3 c00 = tex2D(s_sam, VAR.UL).xyz;
  half3 c20 = tex2D(s_sam, VAR.UR).xyz;
  half3 c02 = tex2D(s_sam, VAR.DL).xyz;
  half3 c22 = tex2D(s_sam, VAR.DR).xyz;
	
  half3 dt = float3(1,1,1);
  
  half k00=dot(abs(c00-c11),dt);
  half k20=dot(abs(c20-c11),dt);
  half k02=dot(abs(c02-c11),dt);
  half k22=dot(abs(c22-c11),dt);

  half md1 = dot(abs(c00-c22),dt);
  half md2 = dot(abs(c02-c20),dt);

  half w1 = md2*k22;half w2 = md1*k02;
  half w3 = md2*k00;half w4 = md1*k20;

  half t1 = w1+w3;
  half t2 = w2+w4;

  half ww = max(t1,t2)+0.001;

  return float4((w1*c00+w2*c20+w3*c22+w4*c02+ww*c11)/(t1+t2+ww),0);
}


technique GS2x_SHADER
{
   pass P0
   {
     VertexShader = compile vs_2_0 GS_VERTEX();
     PixelShader  = compile ps_2_0 GS_FRAGMENT();
   }  
}
