// GS4xColorFilter shader - by guest(r)
// guest.r@gmail.com
// license: GNU-GPL

//*******SHADER SETTINGS**********//

// color params

half3  c_ch = half3 (1.0, 1.0, 1.0); // color channel (r,g,b) intensity
half      a =  1.100;                // saturation 
half      b =  1.000;                // brightness
half      c =  1.270;                // contrast

// filter params

half start_weigth =  0.325;
half            k = -0.250;
half   min_weigth = -0.070;
half   max_weigth =  0.250;
half      lum_add =  0.200; 


// contrast func.

float contrast(float x)
{ return clamp(0.866 + c*(x-0.866),0.05, 1.73); }


// matrix definitions and sampler init 

half4x4 World			: WORLD;
half4x4 View			: VIEW;
half4x4 Projection		: PROJECTION;
half4x4 WorldView		: WORLDVIEW;			
half4x4 ViewProjection		: VIEWPROJECTION;		
half4x4 WorldViewProjection	: WORLDVIEWPROJECTION;		

float2 ps  : TEXELSIZE = 1.0F/256.0F;
half3 dt = half3(1,1,1);
half4 d4 = half4(1,1,1,1);
half  r3 =  1.732051;   


texture s_tex	: SOURCETEXTURE;
sampler	s_sam = sampler_state
{
 Texture   = (s_tex);
 MinFilter = POINT;
 MagFilter = POINT;
};


texture t_tex : WORKINGTEXTURE;
sampler	t_sam = sampler_state
{
 Texture   = (t_tex);
 MinFilter = POINT;
 MagFilter = POINT;
};



string name : NAME = "GS4xColorFilter";


string preprocessTechique : PREPROCESSTECHNIQUE = "COLOR";
string combineTechique : COMBINETECHNIQUE =  "GS4xFILTER";



// vertex shader linkin' structure

struct VERTEX_STUFF // gs4xfilter
{
  float4  coord : POSITION;
  float2  CT    : TEXCOORD0;
  float2  UL    : TEXCOORD1;
  float2  UR    : TEXCOORD2;
  float2  DL    : TEXCOORD3;
  float2  DR    : TEXCOORD4;
  float2  iUL   : TEXCOORD5;
  float2  iUR   : TEXCOORD6;
  float4  iDD   : TEXCOORD7;
};


struct VERTEX_STUFF1 // color
{
  float4  coord : POSITION;
  float2  CT   : TEXCOORD0;
};


// **VS**

// gs4xfilter

VERTEX_STUFF GS4X_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF OUT = (VERTEX_STUFF)0;
  
  float dx = ps.x*0.5;
  float dy = ps.y*0.5;
  float sx = ps.x*0.25;
  float sy = ps.y*0.25;

  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  OUT.UL = tc + float2(-dx,-dy);
  OUT.UR = tc + float2( dx,-dy);
  OUT.DL = tc + float2(-dx, dy);
  OUT.DR = tc + float2( dx, dy);
  
  OUT.iUL = tc + float2(-sx,-sy);
  OUT.iUR = tc + float2( sx,-sy);
  OUT.iDD = float4(tc,tc) + float4(-sx, sy, sx, sy);
  
return OUT;

}

// color

VERTEX_STUFF1 COLOR_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF1 OUT = (VERTEX_STUFF1)0;
  
  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  return OUT;
}

// **PS**

// GS
float4 GS4X_FRAGMENT ( in VERTEX_STUFF VAR ) : COLOR

{
  half3 c  = tex2D(t_sam, VAR.CT).xyz;
  half3 o1 = tex2D(t_sam, VAR.UL).xyz;
  half3 o2 = tex2D(t_sam, VAR.UR).xyz;
  half3 o3 = tex2D(t_sam, VAR.DR).xyz;
  half3 o4 = tex2D(t_sam, VAR.DL).xyz;
  half3 i1 = tex2D(t_sam, VAR.iUL).xyz;
  half3 i2 = tex2D(t_sam, VAR.iUR).xyz;
  half3 i3 = tex2D(t_sam, VAR.iDD.zw).xyz;
  half3 i4 = tex2D(t_sam, VAR.iDD.xy).xyz;

  half ko1=dot(abs(o1-c),dt);
  half ko2=dot(abs(o2-c),dt);
  half ko3=dot(abs(o3-c),dt);
  half ko4=dot(abs(o4-c),dt);

  half sd1 = dot(abs(i1-i3),dt);
  half sd2 = dot(abs(i2-i4),dt);
  
  half4 w = half4(sd2,sd1,sd2,sd1);

  if (ko3<ko1) w.x = 0.0;
  if (ko4<ko2) w.y = 0.0;
  if (ko1<ko3) w.z = 0.0;
  if (ko2<ko4) w.w = 0.0;  

  c = (w.x*o1+w.y*o2+w.z*o3+w.w*o4+0.1*c)/(dot(w,d4)+0.1);
  
  w.z = k/(0.4*dot(c,dt)+lum_add);
 
  w.x = clamp(w.z*sd1+start_weigth,min_weigth,max_weigth); 
  w.y = clamp(w.z*sd2+start_weigth,min_weigth,max_weigth);

  return half4((w.x*(i1+i3) + w.y*(i2+i4) + (1.0-2.0*(w.x+w.y))*c),0);

}

// cOLOR

float4 COLOR_FRAGMENT ( in VERTEX_STUFF1 VAR ) : COLOR

{
  half3 color = tex2D(s_sam, VAR.CT).xyz;
  
  half x  = sqrt(dot(color,color));   
  color.r = pow(color.r,a);
  color.g = pow(color.g,a);
  color.b = pow(color.b,a);
  return half4(contrast(x)*normalize(color*c_ch)*b,1);
}


technique COLOR
{
   pass P0
   {
     VertexShader = compile vs_2_0 COLOR_VERTEX();
     PixelShader  = compile ps_2_0 COLOR_FRAGMENT();
     Sampler[0] = (s_sam);
   }  
}


technique GS4xFILTER
{
   pass P0
   {
     VertexShader = compile vs_2_0 GS4X_VERTEX();
     PixelShader  = compile ps_2_0 GS4X_FRAGMENT();
     Sampler[0] = (t_sam);
   }  
}
