// GS4xScale shader - by guest(r)
// guest.r@gmail.com
//
// license: GNU-GPL
//
//
// ***DosBox***
//
// Best use it in the fullscreen mode (works with fullfixed=true, scaler=normal2x)
// the output should be set to direct3d.
//
// Best fullscreen modes are 4x multiplies of the native "game" mode (usually 4x(320,200)).
// The output image might also look nice with anything near (like 1280x960...), looks nicer
// when aspect is set to false.



// definitions and sampler init 

half4x4 World			: WORLD;
half4x4 View			: VIEW;
half4x4 Projection		: PROJECTION;
half4x4 WorldView		: WORLDVIEW;			
half4x4 ViewProjection		: VIEWPROJECTION;		
half4x4 WorldViewProjection	: WORLDVIEWPROJECTION;		

float2 ps  : TEXELSIZE = 1.0F/256.0F;
half3 dt = half3(1,1,1);
half4 d4 = half4(1,1,1,1);

texture s_tex	: SOURCETEXTURE;

sampler	s_sam = sampler_state
{
 Texture   = (s_tex);
 MinFilter = POINT;
 MagFilter = POINT;
};


string name : NAME = "GS4xScale";

string combineTechique : COMBINETECHNIQUE =  "GS4xScale_SHADER";

// vertex shader linkin' structure

struct VERTEX_STUFF
{
  float4  coord : POSITION;
  float2  CT    : TEXCOORD0;
  float2  UL    : TEXCOORD1;
  float2  UR    : TEXCOORD2;
  float2  DL    : TEXCOORD3;
  float2  DR    : TEXCOORD4;
  float2  iUL   : TEXCOORD5;
  float2  iUR   : TEXCOORD6;
  float4  iDD   : TEXCOORD7;

};

// **VS**

VERTEX_STUFF GS_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF OUT = (VERTEX_STUFF)0;
  
  float dx = ps.x*0.5;
  float dy = ps.y*0.5;
  float sx = ps.x*0.25;
  float sy = ps.y*0.25;

  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  OUT.UL = tc + float2(-dx,-dy);
  OUT.UR = tc + float2( dx,-dy);
  OUT.DL = tc + float2(-dx, dy);
  OUT.DR = tc + float2( dx, dy);
  
  OUT.iUL = tc + float2(-sx,-sy);
  OUT.iUR = tc + float2( sx,-sy);
  OUT.iDD = float4(tc,tc) + float4(-sx, sy, sx, sy);
  
  return OUT;
}

// **PS**

float4 GS_FRAGMENT ( in VERTEX_STUFF VAR ) : COLOR

{
  half3 c  = tex2D(s_sam, VAR.CT).xyz;
  half3 o1 = tex2D(s_sam, VAR.UL).xyz;
  half3 o2 = tex2D(s_sam, VAR.UR).xyz;
  half3 o3 = tex2D(s_sam, VAR.DR).xyz;
  half3 o4 = tex2D(s_sam, VAR.DL).xyz;
  half3 i1 = tex2D(s_sam, VAR.iUL).xyz;
  half3 i2 = tex2D(s_sam, VAR.iUR).xyz;
  half3 i3 = tex2D(s_sam, VAR.iDD.zw).xyz;
  half3 i4 = tex2D(s_sam, VAR.iDD.xy).xyz;

  half ko1=dot(abs(o1-c),dt);
  half ko2=dot(abs(o2-c),dt);
  half ko3=dot(abs(o3-c),dt);
  half ko4=dot(abs(o4-c),dt);

  half k1=dot(abs(i1-i3),dt)*dot(abs(o1-o3),dt);
  half k2=dot(abs(i2-i4),dt)*dot(abs(o2-o4),dt);
  
  half4 w = half4(k2,k1,k2,k1);

  if (ko3<ko1) w.x = 0.0;
  if (ko4<ko2) w.y = 0.0;
  if (ko1<ko3) w.z = 0.0;
  if (ko2<ko4) w.w = 0.0;  

  return half4((w.x*o1+w.y*o2+w.z*o3+w.w*o4+0.001*c)/(dot(w,d4)+0.001),0);
}


technique GS4xScale_SHADER
{
    pass P0
    {
	VertexShader = compile vs_2_0 GS_VERTEX();
	PixelShader  = compile ps_2_0 GS_FRAGMENT();
    }  
}
